@@ -1,4 +1,5 @@
 //! Common types shared between the encoder and decoder
+use crate::text_metadata::{EncodableTextChunk, ITXtChunk, TEXtChunk, ZTXtChunk};
 use crate::{chunk, encoder};
 use io::Write;
 use std::{borrow::Cow, convert::TryFrom, fmt, io};
@@ -314,10 +315,24 @@ pub enum Compression {
     /// the encoder can do, but is meant to emulate the `Best` setting in the `Flate2`
     /// library.
     Best,
+    #[deprecated(
+        since = "0.17.6",
+        note = "use one of the other compression levels instead, such as 'fast'"
+    )]
     Huffman,
+    #[deprecated(
+        since = "0.17.6",
+        note = "use one of the other compression levels instead, such as 'fast'"
+    )]
     Rle,
 }
 
+impl Default for Compression {
+    fn default() -> Self {
+        Self::Default
+    }
+}
+
 /// An unsigned integer scaled version of a floating point value,
 /// equivalent to an integer quotient with fixed denominator (100_000)).
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
@@ -350,14 +365,12 @@ impl ScaledFloat {
     /// Slightly inaccurate scaling and quantization.
     /// Clamps the value into the representable range if it is negative or too large.
     pub fn new(value: f32) -> Self {
-        Self {
-            0: Self::forward(value),
-        }
+        Self(Self::forward(value))
     }
 
     /// Fully accurate construction from a value scaled as per specification.
     pub fn from_scaled(val: u32) -> Self {
-        Self { 0: val }
+        Self(val)
     }
 
     /// Get the accurate encoded value.
@@ -367,7 +380,7 @@ impl ScaledFloat {
 
     /// Get the unscaled value as a floating point.
     pub fn into_value(self) -> f32 {
-        Self::reverse(self.0) as f32
+        Self::reverse(self.0)
     }
 
     pub(crate) fn encode_gama<W: Write>(self, w: &mut W) -> encoder::Result<()> {
@@ -459,6 +472,7 @@ impl SrgbRenderingIntent {
 
 /// PNG info struct
 #[derive(Clone, Debug)]
+#[non_exhaustive]
 pub struct Info<'a> {
     pub width: u32,
     pub height: u32,
@@ -469,14 +483,23 @@ pub struct Info<'a> {
     /// The image's `tRNS` chunk, if present; contains the alpha channel of the image's palette, 1 byte per entry.
     pub trns: Option<Cow<'a, [u8]>>,
     pub pixel_dims: Option<PixelDimensions>,
-    /// Gamma of the source system.
-    pub source_gamma: Option<ScaledFloat>,
     /// The image's `PLTE` chunk, if present; contains the RGB channels (in that order) of the image's palettes, 3 bytes per entry (1 per channel).
     pub palette: Option<Cow<'a, [u8]>>,
+    /// The contents of the image's gAMA chunk, if present.
+    /// Prefer `source_gamma` to also get the derived replacement gamma from sRGB chunks.
+    pub gama_chunk: Option<ScaledFloat>,
+    /// The contents of the image's `cHRM` chunk, if present.
+    /// Prefer `source_chromaticities` to also get the derived replacements from sRGB chunks.
+    pub chrm_chunk: Option<SourceChromaticities>,
+
     pub frame_control: Option<FrameControl>,
     pub animation_control: Option<AnimationControl>,
     pub compression: Compression,
+    /// Gamma of the source system.
+    /// Set by both `gAMA` as well as to a replacement by `sRGB` chunk.
+    pub source_gamma: Option<ScaledFloat>,
     /// Chromaticities of the source system.
+    /// Set by both `cHRM` as well as to a replacement by `sRGB` chunk.
     pub source_chromaticities: Option<SourceChromaticities>,
     /// The rendering intent of an SRGB image.
     ///
@@ -484,8 +507,12 @@ pub struct Info<'a> {
     pub srgb: Option<SrgbRenderingIntent>,
     /// The ICC profile for the image.
     pub icc_profile: Option<Cow<'a, [u8]>>,
-    /// Private field to mark the struct as non-exhaustive.
-    _extensible: (),
+    /// tEXt field
+    pub uncompressed_latin1_text: Vec<TEXtChunk>,
+    /// zTXt field
+    pub compressed_latin1_text: Vec<ZTXtChunk>,
+    /// iTXt field
+    pub utf8_text: Vec<ITXtChunk>,
 }
 
 impl Default for Info<'_> {
@@ -498,17 +525,21 @@ impl Default for Info<'_> {
             interlaced: false,
             palette: None,
             trns: None,
+            gama_chunk: None,
+            chrm_chunk: None,
             pixel_dims: None,
-            source_gamma: None,
             frame_control: None,
             animation_control: None,
             // Default to `deflate::Compression::Fast` and `filter::FilterType::Sub`
             // to maintain backward compatible output.
             compression: Compression::Fast,
+            source_gamma: None,
             source_chromaticities: None,
             srgb: None,
             icc_profile: None,
-            _extensible: (),
+            uncompressed_latin1_text: Vec::new(),
+            compressed_latin1_text: Vec::new(),
+            utf8_text: Vec::new(),
         }
     }
 }
@@ -608,6 +639,17 @@ impl Info<'_> {
         data[9] = self.color_type as u8;
         data[12] = self.interlaced as u8;
         encoder::write_chunk(&mut w, chunk::IHDR, &data)?;
+        // Encode the pHYs chunk
+        if let Some(pd) = self.pixel_dims {
+            let mut phys_data = [0; 9];
+            phys_data[0..4].copy_from_slice(&pd.xppu.to_be_bytes());
+            phys_data[4..8].copy_from_slice(&pd.yppu.to_be_bytes());
+            match pd.unit {
+                Unit::Meter => phys_data[8] = 1,
+                Unit::Unspecified => phys_data[8] = 0,
+            }
+            encoder::write_chunk(&mut w, chunk::pHYs, &phys_data)?;
+        }
 
         if let Some(p) = &self.palette {
             encoder::write_chunk(&mut w, chunk::PLTE, p)?;
@@ -635,6 +677,19 @@ impl Info<'_> {
         if let Some(actl) = self.animation_control {
             actl.encode(&mut w)?;
         }
+
+        for text_chunk in &self.uncompressed_latin1_text {
+            text_chunk.encode(&mut w)?;
+        }
+
+        for text_chunk in &self.compressed_latin1_text {
+            text_chunk.encode(&mut w)?;
+        }
+
+        for text_chunk in &self.utf8_text {
+            text_chunk.encode(&mut w)?;
+        }
+
         Ok(())
     }
 }
@@ -683,13 +738,15 @@ bitflags! {
     "]
     pub struct Transformations: u32 {
         /// No transformation
-        const IDENTITY            = 0x0000; // read and write */
+        const IDENTITY            = 0x00000; // read and write */
         /// Strip 16-bit samples to 8 bits
-        const STRIP_16            = 0x0001; // read only */
+        const STRIP_16            = 0x00001; // read only */
         /// Expand paletted images to RGB; expand grayscale images of
         /// less than 8-bit depth to 8-bit depth; and expand tRNS chunks
         /// to alpha channels.
-        const EXPAND              = 0x0010; // read only */
+        const EXPAND              = 0x00010; // read only */
+        /// Expand paletted images to include an alpha channel. Implies `EXPAND`.
+        const ALPHA               = 0x10000; // read only */
     }
 }
 
